
/* ---------------------------------------------INDEXER------------------------------------------------------------------- */

// ----------OUTPUTS----------------

const unsigned int bLiftValve  = 0x00010000; //set bit16 - OUT1 on the ICU-IDX Board  -  Lift Valve
const unsigned int bIdxValve   = 0x00020000; //set bit17 - OUT2 on the ICU-IDX Board  -  Indexer Lock Valve
const unsigned int bFlash1     = 0x00040000; //set bit18 - OUT3 on the ICU-IDX Board  -  Flash 1 output
const unsigned int bFlash2     = 0x00080000; //set bit19 - OUT4 on the ICU-IDX Board  -  Flash 2 output
const unsigned int bFlash3     = 0x00100000; //set bit20 - OUT5 on the ICU-IDX Board  -  Flash 3 output
const unsigned int bFlash4     = 0x00200000; //set bit21 - OUT6 on the ICU-IDX Board  -  Flash 4 output

// ----------INPUTS-----------------

const unsigned char bLiftDn     = 0x01; //set bit0 - INP1 on the ICU-IDX Board  -  Lift Down PRX
const unsigned char bLiftUp     = 0x02; //set bit1 - INP2 on the ICU-IDX Board  -  Lift Up PRX
const unsigned char bIdxLock    = 0x04; //set bit2 - INP3 on the ICU-IDX Board  -  Indexer Lock PRX
const unsigned char bFreeWheel  = 0x08; //set bit3 - INP4 on the ICU-IDX Board  -  Free Wheel PRX

/* ----------------------------------------------------------------------------------------------------------------------- */




/* ----------------------------------------------HEAD--------------------------------------------------------------------- */

// ----------OUTPUTS----------------

const unsigned int bSqueeValve = 0x00010000; //set bit16 - OUT1 on the ICU-HEAD Board  -  Squeegee Valve
const unsigned int bFloodValve = 0x00020000; //set bit17 - OUT2 on the ICU-HEAD Board  -  Flood Valve
const unsigned int bScreenL    = 0x00040000; //set bit18 - OUT3 on the ICU-HEAD Board  -  Screen Left Valve
const unsigned int bScreenR    = 0x00080000; //set bit19 - OUT4 on the ICU-HEAD Board  -  Screen Right Valve
const unsigned int bSafetyLED  = 0x00100000; //set bit20 - OUT5 on the ICU-HEAD Board  -  Red LED on the head (Safety LED)
const unsigned int bBuzzer     = 0x00200000; //set bit21 - OUT6 on the ICU-HEAD Board  -  Buzzer

// ----------INPUTS----------------

const unsigned int bSafetyBar  = 0x01; //set bit0 - INP1 on the ICU-HEAD Board  -  Safety Bar
//const unsigned int bSqueeValve = 0x02; //set bit17 - INP2 on the ICU-HEAD Board
//const unsigned int bFloodValve = 0x04; //set bit18 - INP3 on the ICU-HEAD Board
//const unsigned int bBuzer      = 0x08; //set bit19 - INP4 on the ICU-HEAD Board
//const unsigned int bSqueeValve = 0x10; //set bit20 - INP5 on the ICU-HEAD Board
//const unsigned int bSqueeValve = 0x20; //set bit21 - INP6 on the ICU-HEAD Board

/* ----------------------------------------------------------------------------------------------------------------------- */


/* ---------------------MODES OF ICU--------------------------------------------------------------------------------------- */

const char ICU_OPMODE_PTP = 0x01;
const char ICU_OPMODE_JOG = 0x02;
const char IDX_BOARD      = 0x00;

//CONTROLWORD OF ICU
const unsigned short ICU_CTRL_DISABLE = 0x0006;
const unsigned short ICU_CTRL_SWITCH_ON = 0x0007;
const unsigned short ICU_CTRL_ENABLE = 0x000F;
const unsigned short ICU_CTRL_MA = 0x000F;
const unsigned short ICU_CTRL_MI = 0x004F;
const unsigned short ICU_CTRL_HOME = 0x001F;

//STATUSWORD OF ICU
const unsigned short ICU_STAT_RS = 0x1000;
const unsigned short ICU_STAT_HOME_OK = 0x1000;
const unsigned short ICU_STAT_HOME_ERR = 0x2000;
const unsigned short ICU_STAT_FAULT = 0x0008;
const unsigned short ICU_STAT_MOVE_DONE = 0x0400;

//NMT SERVICE
const unsigned char ICU_PREOPERATION_MODE = 0x00;
const unsigned char ICU_OPERATION_MODE    = 0x01;

//PDO1:
//FROM RIGHT TO LEFT:
//	BYTE0 -> INPUTS
//	BYTE1 -> KEYPAD COLUMN4
//	BYTE2 -> KEYPAD COLUMN3
//	BYTE3 -> KEYPAD COLUMN2
//	BYTE4 -> KEYPAD COLUMN1
//	BYTE5 -> KEYPAD COLUMN0
//	BYTE6 -> STATUSWORD LSB BYTE
//	BYTE7 -> STATUSWORD MSB BYTE


//KEYPAD - VINDICATOR - C1-C4, R0-R4  -> PDO1: BYTE1-BYTE4
#define PRINT_VEL_UP 0x10000000
#define PRINT_VEL_DN 0x08000000
#define FLOOD_VEL_UP 0x00100000
#define FLOOD_VEL_DN 0x00080000
#define POS_FR_UP    0x00001000
#define POS_FR_DN    0x00000800
#define POS_RR_UP    0x00000010
#define POS_RR_DN    0x00000008
#define JOG_UP       0x00010000
#define JOG_DN       0x01000000
#define AIR_LEFT     0x00000200
#define AIR_RIGHT    0x00000002
#define KEY_SET      0x02000000
#define KEY_TEST     0x00020000
#define KEY_STOP     0x04000000
#define KEY_IDX      0x00040000
#define KEY_TABLE    0x00000400
#define KEY_SQ_FL    0x00000004
#define NO_KEY	     0x00000000

/*
//KEYPAD - TRIDENT - C0-C3, R0-R4  -> PDO1: BYTE2-BYTE5
#define PRINT_VEL_UP 0x01000000
#define PRINT_VEL_DN 0x02000000
#define FLOOD_VEL_UP 0x00010000
#define FLOOD_VEL_DN 0x00020000
#define POS_FR_UP    0x00000100
#define POS_FR_DN    0x00000200
#define POS_RR_UP    0x00000001
#define POS_RR_DN    0x00000002
#define JOG_UP       0x00010004
#define JOG_DN       0x01000008
#define IDX_LEFT     0x00001000
#define IDX_RIGHT    0x00000010
#define KEY_F1       0x00100000
#define KEY_TEST     0x00080000
#define KEY_STOP     0x10000000
#define KEY_WHEEL    0x00000800
#define KEY_TABLE    0x00000400
#define KEY_ON_OFF   0x08000000
#define KEY_SQUEE    0x00040000
#define KEY_FLOOD    0x04000000

//KEYPAD - LEDS - TRIDENT
const unsigned short KLED_ON_OFF = 0x0001;
const unsigned short KLED_F1     = 0x0002;
const unsigned short KLED_TABLE  = 0x0800;
const unsigned short KLED_WHEEL  = 0x0400;
*/

char key_jogup_pressed[22];
char key_jogdn_pressed[22];
char key_airll_pressed[22];
char key_airlr_pressed[22];
char key_test_pressed[22];
char key_stop_pressed;
char key_index_pressed[22];
char key_table_pressed;
char key_squ_fl_pressed[22];
char key_set_pressed[22];









/*


void move_lift_down();
void move_lift_up();
void move_idx_lock();
void move_idx_unlock();
void move_idx_posA();  //move to position near home position
void move_idx_posB();  //move to postion away from home position
void move_idx_posAB();  //move to medium position (HALF INDEX)
void move_head_posA(int head_no);  //move to outside position (front position)
void move_head_posB(int head_no);  //move to inside position (rear position)
void move_flood_down(int head_no);
void move_flood_up(int head_no);
void move_squee_down(int head_no);
void move_squee_up(int head_no);
void buzzer_on(unsigned long buzz_time);
void switch_LED(unsigned char state, int head_no);
void machine_error();
void head_home(int head_no);
void idx_home();
void start_heat(int head_no);
void stop_heat(int head_no);
*/
/* --------------------------------------------------------------------------------------------------- */
/* ---------------------  START CAN OPEN PROCEDURES  ------------------------------------------------- */
/* --------------------------------------------------------------------------------------------------- */
/*
unsigned short check_ICU_status(unsigned char head_no);
void change_POS(int head_no, int param_val);
void change_VEL(int head_no, unsigned int param_val);
void change_ACC(int head_no, unsigned int param_val);
void change_DACC(int head_no, unsigned int param_val);
void change_H_OFF(int head_no, int param_val);
void change_H_VEL(int head_no, unsigned int param_val);
void change_H_ACC(int head_no, unsigned int param_val);
void change_CTRL(int head_no, unsigned short param_val);
void change_MODE(int head_no, char param_val);
void change_OUT(int head_no, unsigned int param_val);	//bit16 - OUT1, ... , bit23 - OUT8
void write_STRING(int head_no, char data[]);
unsigned int read_OUT(int head_no);
unsigned int read_INP(int head_no);	//bit16 - INP1, ... , bit21 - INP6
*/
/* ----------------------------------------------------------------------------------------------------------------- */
/* ---------------------------- END CAN OPEN PROCEDURES ------------------------------------------------------------ */
/* ----------------------------------------------------------------------------------------------------------------- */


