#!/bin/bash

#-----------------------------------------------------
# Set time and date
#----------------------------------------------------
time_and_date()
{
   clear;
   echo "Please enter new time and date in month,day,hour,minute,year format "
   echo -n "(mmddhhmmyyyy) :"
   read TIMEDATE

   echo -n "Time and date set to ";
   date -s $TIMEDATE

   sleep 2
}

#-----------------------------------------------------
# Menu for configuring serial devices
#----------------------------------------------------
serial_config()
{
while [ 1 ]; do

clear
echo
echo "                SIB serial port configuration"
echo	
echo
echo "      1 ) Configure SLIP port"
echo "      2 ) Configure terminal port"
echo "      3 ) Configure dial-in terminal port"
echo "      4 ) Configure dial-in PPP port"
echo "      5 ) Configure dial-out PPP connection"
echo "      6 ) Configure standard serial port"
echo "      7 ) View serial setting"
echo
echo "  b) Back to main menu"
echo "  ----------------------------------------------------------------------------"
echo -n "	Selection: "

read OPT


	case $OPT in
		b | B)	echo;echo "Returning to main menu...";echo
			break ;;

		    1 )	echo -en "\n\n\tDevice (ttyS*) number to configure (starting from 0)? "
		        read PORTNUM
			serial_ip_config  $PORTNUM
			;;


		    2 )	echo -en "\n\n\tDevice (ttyS*) number to configure (starting from 0)? "
		        read PORTNUM
			serial_term_config  $PORTNUM
			;;

		    3 ) echo -en "\n\n\tDevice (ttyS*) number to configure (starting from 0)? "
		        read PORTNUM
			serial_dialup_term  $PORTNUM
			;;

		    4 ) echo -en "\n\n\tDevice (ttyS*) number to configure (starting from 0)? "
		        read PORTNUM
			dialin_ppp_config  $PORTNUM
			;;

		    5 ) echo; echo; 
			dialout_ppp_config
			;;

		    6 ) echo -en "\n\n\tDevice (ttyS*) number to configure (starting from 0)? "
		        read PORTNUM
			serial_raw_config $PORTNUM
			;;

		    7 ) echo -en "\n\n\tDevice (ttyS*) number to view (starting from 0)? "
		        read PORTNUM
			serial_view_info $PORTNUM
			;;

	esac

done

}

#----------------------------------------------------
# Dial-in and out + PPP functions
#----------------------------------------------------
# Function to setup an mgetty on a port

serial_dialup_term()
{
    typeset -i k=0
    LINESPEED="0"
    clear
    echo
    echo "Configuring dial-in terminal device ttyS${1}"
    DEVICE="ttyS${1}"
    ITLINE="$(grep $DEVICE /etc/inittab | grep mgetty | grep -v \#)"

    for i in $ITLINE
    do

	if [ "${k}" = "4" ]
	then
	    LINESPEED=$i
	fi

	k=${k}+1

    done
 
    echo 
    echo "Current setting for dial-in terminal ${DEVICE}:"
    echo "Line Speed = ${LINESPEED}"
    echo
    echo -n "New line speed (38400)? "
    read LINESPEED


    if [ "${LINESPEED}" = "" ]
    then
	LINESPEED="38400"
    fi

    echo 
    echo "New setting for dial-in terminal ${DEVICE}:"
    echo "Line Speed = ${LINESPEED}"
    echo
    
    echo -n "Update dial-in terminal configuration for ttyS${1} (y/N) ?"
    read TEMP

    if [ "$TEMP" = "y" -o "$TEMP" = "Y" ]
    then
	echo "Updating dial-in terminal configuration...."
	# remove any old terminal entry from inittab, getty or mgetty
	grep -v ttyS${1} /etc/inittab > /tmp/inittemp
	cp /tmp/inittemp /etc/inittab
	rm /tmp/inittemp
	# remove any slip configuration file
	rm -rf /etc/sysconfig/serial${1}ip
	# remove any raw configuration script
	rm -rf /etc/sysconfig/serial${1}raw
	# Add the new terminal line
	echo "S${1}:23456:respawn:/sbin/mgetty -x0 -D -s ${LINESPEED} -m '\"\" ATZ OK' ttyS${1}" >> /etc/inittab
	echo "Dial-in terminal configuration updated."
    else
	echo
	echo "Dial-in terminal configuration NOT updated."
    fi

sleep 1
}


#----------------------------------------------------
# Function to setup a options.ttySx file

dialin_ppp_config()
{
    clear
    echo
    echo "Configuring dial-in PPP server for device ttyS${1}"
    echo
    DEVICE="ttyS${1}"
    echo "Current IP setting for PPP server ${DEVICE}, local:remote addresses:"

    if [ -f /etc/ppp/options.ttyS${1} ]
    then
	cat /etc/ppp/options.ttyS${1}
    else
	echo "NONE"
    fi

    echo
    ITLINE="$(grep $DEVICE /etc/inittab | grep mgetty | grep -v \#)"

    if [ "${ITLINE}" = "" ]
    then
	echo "Device is NOT configured as a dial-in device." 
    else
	echo "Device is configured for dial-in."
    fi

    echo
    echo -n "New local IP address (0.0.0.0)? "
    read LOCIP
    echo -n "New remote IP address (0.0.0.0)? "
    read RMTIP

    if [ "${LOCIP}" = "" ]
    then
	LOCIP="0.0.0.0"
    fi

    if [ "${RMTIP}" = "" ]
    then
	RMTIP="0.0.0.0"
    fi

    echo 
    echo "New settings, local:remote IP:"
    echo "${LOCIP}:${RMTIP}"

    echo -n "Update PPP server configuration for ttyS${1} (y/N) ?"
    read TEMP

    if [ "$TEMP" = "y" -o "$TEMP" = "Y" ]
    then
	if [ "${ITLINE}" = "" ]
	then
	    echo
	    echo "Warning: this device will not accept PPP connections"
	    echo "until it has also been configured as a dial-in terminal"
	    echo "device."
	fi
	
	echo
	echo "Updating PPP server configuration..."
	echo "${LOCIP}:${RMTIP}" > /etc/ppp/options.ttyS${1}
	echo "PPP dial-in configuration updated."
    else
	echo
	echo "PPP server configuration NOT updated."
    fi

sleep 1
}

#-----------------------------------------------------
# Function to generate dial-out script ppp-on-ttySx

dialout_ppp_config()
{
    echo -n "Connection identifier? "
    read CONIDENT
    clear
    echo
    echo "Configuring dial-out PPP connection for connection ${CONIDENT}"
    echo 
    echo "Current setting for dial-out PPP connection ${CONIDENT}:"

    if [ -f /etc/ppp/ppp-${CONIDENT} ]
    then
	cat /etc/ppp/ppp-${CONIDENT} | grep -v \# | grep -v export | grep -v SCRIPT
    else
	echo "NOT CONFIGURED"
    fi

    echo "Serial tty number to use for this connection" 
    echo -n "(leave blank to delete this script)? "
    read DEVICE

    if [ "${DEVICE}" = "" ]
    then
	# get confirmation, then delete script and exit
	echo -n "Delete script with identifier \"${CONIDENT}\" (y/N)? "
	read TEMP

	if [ "$TEMP" = "y" -o "$TEMP" = "Y" ]
	then
	    echo "Deleting connection script.  This does NOT delete the device-specific configuration."
	    rm -f /etc/ppp/ppp-${CONIDENT}

	    # Make it delete the "run at boot" stuff
	    cat /etc/sysconfig/pppcfg.sh | grep -v "${CONIDENT}" > /tmp/ppptmp
	    mv /tmp/ppptmp /etc/sysconfig/pppcfg.sh
	    chmod 754 /etc/sysconfig/pppcfg.sh

	    sleep 1
	    return
	else
	    echo "Connection script NOT deleted."
	    sleep 1
	    return
	fi
    fi

    DEVICE="ttyS${DEVICE}"

    echo -n "Speed of connection to modem (38400)? "
    read LINESPEED

    if [ "${LINESPEED}" = "" ]
    then
	LINESPEED=38400
    fi

    echo -n "Telephone number to dial? "
    read TELEPHONE
    echo -n "Remote account user name? "
    read ACCOUNT
    echo -n "Remote account password? "
    read PASSWORD
    echo -n "Netmask (optional)? "
    read NETMASK
    echo -n "Command prompt on remote machine (optional)? "
    read RMTPROMPT
    echo -n "Command to begin PPP on remote host (optional)? "
    read PPPSTART
    echo; echo "The following parameters affect ALL dial scripts which use this device"
    echo "as well as dial-in configurations which use this device:"
    echo
    echo -n "Use dial-on-demand?"
    read DEMANDDIAL
    echo -n "Local IP address - 0.0.0.0 to let remote assign (0.0.0.0)? "
    read PPPLOCIP
    echo -n "Remote IP address - required for dial-on-demand (0.0.0.0)? "
    read PPPRMTIP
    echo -n "Start this interface at boot time? "
    read PPPBOOTSTART

    # set some defaults if need be
    if [ "${PPPLOCIP}" = "" ]
    then
	PPPLOCIP="0.0.0.0"
    fi
    if [ "${PPPRMTIP}" = "" ]
    then
	PPPRMTIP="0.0.0.0"
    fi

    echo
    echo "New parameters for connection ${CONIDENT}:"
    echo "Device = ${DEVICE}"
    echo "Line speed = ${LINESPEED}"
    echo "Telephone number = ${TELEPHONE}"
    echo "User name = ${ACCOUNT}"
    echo "Password = ${PASSWORD}"
    echo "Netmask = ${NETMASK}"
    echo "Remote prompt = ${RMTPROMT}"
    echo "PPP command = ${PPPSTART}"
    echo "Addressing:  ${PPPLOCIP}:${PPPRMTIP}"
    if [ "${DEMANDDIAL}" = "Y" -o "${DEMANDDIAL}" = "y" ]
    then
	echo "Dial on demand"
    fi
    if [ "${PPPBOOTSTART}" = "Y" -o "${PPPBOOTSTART}" = "y" ]
    then
	echo "Interface will be started at boot time."
    fi


    echo -n "Create connection script ${CONIDENT} (y/N) ?"
    read TEMP

	if [ "$TEMP" = "y" -o "$TEMP" = "Y" ]
	then
	    echo "Creating connection script."
	    echo \#\!/bin/sh > /etc/ppp/ppp-${CONIDENT}
	    echo "TELEPHONE=${TELEPHONE}" >> /etc/ppp/ppp-${CONIDENT}
	    echo "ACCOUNT=${ACCOUNT}" >> /etc/ppp/ppp-${CONIDENT}
	    echo "PASSWORD=${PASSWORD}" >> /etc/ppp/ppp-${CONIDENT}
	    echo "NETMASK=${NETMASK}" >> /etc/ppp/ppp-${CONIDENT}
	    echo "RMTPROMPT=\"${RMTPROMPT}\"" >> /etc/ppp/ppp-${CONIDENT}
	    echo "PPPSTART=\"${PPPSTART}\"" >> /etc/ppp/ppp-${CONIDENT}
	    echo "export TELEPHONE ACCOUNT PASSWORD RMTPROMPT PPPSTART" >> /etc/ppp/ppp-${CONIDENT}
	    echo "DIALER_SCRIPT=/usr/bin/ppp-on-dialer" >> /etc/ppp/ppp-${CONIDENT}
	    echo "exec /usr/sbin/pppd ${DEVICE} ${LINESPEED} connect \$DIALER_SCRIPT" >> /etc/ppp/ppp-${CONIDENT}
  
	    chmod 755 /etc/ppp/ppp-${CONIDENT}

	    echo "Setting device parameters."
	    echo "${PPPLOCIP}:${PPPRMTIP}" > /etc/ppp/options.${DEVICE}
	    if [ "${DEMANDDIAL}" = "Y" -o "${DEMANDDIAL}" = "y" ]
	    then
		echo "demand" >> /etc/ppp/options.${DEVICE}
	    fi

	    if [ "${PPPBOOTSTART}" = "Y" -o "${PPPBOOTSTART}" = "y" ]
	    then
		echo "Setting interface to start at boot."
		echo "/etc/ppp/ppp-${CONIDENT}" >> /etc/sysconfig/pppcfg.sh
	    fi


	else
	    echo "Connection script NOT created or modified."
	fi

sleep 1
}

#-----------------------------------------------------
# Function to view a serial port's information
#-----------------------------------------------------

serial_view_info()
{
    clear
    CONFIGURED=""
    DEVICE="ttyS${1}"
    typeset -i k=0

# First check if configred as a terminal
echo "Checking if ${DEVICE} is configured as a terminal server..."

    ITLINE="$(grep $DEVICE /etc/inittab | grep -v mgetty | grep -v \#)"

    if [ "$ITLINE" != "" ]
    then
	CONFIGURED="1"
	for i in $ITLINE
	do
	    if [ "${k}" = "3" ]
	    then
		LINESPEED="${i}"
	    fi

	    if [ "${k}" = "4" ]
	    then
		TERMEMUL="${i}"
	    fi

	    k=${k}+1
	done

	echo "Port /dev/ttyS${1} is configured as a serial terminal server."
	echo "Line speed = ${LINESPEED}"
	echo "Terminal emulation = ${TERMEMUL}"
    else
	echo "No."
    fi

    echo
    echo "Press enter to continue."
    read TEMP

# Check if a dial-up line
echo "Checking if ${DEVICE} is configured as a dial-in terminal..."
k=0

    ITLINE="$(grep $DEVICE /etc/inittab | grep mgetty | grep -v \#)"

    if [ "$ITLINE" != "" ]
    then
	CONFIGURED="1"
	for i in $ITLINE
	do
	    if [ "${k}" = "4" ]
	    then
		LINESPEED="${i}"
	    fi

	    k=${k}+1
	done

	echo "Port /dev/ttyS${1} is configured as a dial-in terminal."
	echo "Line speed = ${LINESPEED}"
    else
	echo "No."
    fi

    echo
    echo "Press enter to continue."
    read TEMP

# Check if there is a line-specific PPP options file
echo
echo "Checking if ${DEVICE} has a port-specific PPP configuration file..."

if [ -f /etc/ppp/options.ttyS${1} ]
then
    echo "Yes.  PPP IP configuration, local:remote, is:"
    cat /etc/ppp/options.ttyS${1}
else
    echo "No."
fi

    echo
    echo "Press enter to continue."
    read TEMP


# Check if there is a dial-out PPP script file
echo
echo "Checking if ${DEVICE} has a PPP dial-out script..."
FOUNDDIALER="0"

    for j in $(ls /etc/ppp/ppp-* | grep -v dialer | grep -v on | grep -v off)
    do
	TTYSEARCH="$(cat ${j} | grep ttyS${1})"

	if [ "${TTYSEARCH}" != "" ]
	then
	    echo "Found a ppp dialer script \"${j}\" for this device"
	    FOUNDDIALER="1"
	fi
    done

if [ "${FOUNDDIALER}" = "0" ]
then
    echo "No."
fi
    
    echo
    echo "Press enter to continue."
    read TEMP


# Next check if configured for SLIP
echo
echo "Checking if ${DEVICE} is configured as SLIP port..."
DEVBAK=${DEVICE}

if [ -f /etc/sysconfig/serial?ip ]
then

    for j in $(ls /etc/sysconfig/serial?ip)
    do
	. ${j}
	if [ "$DEVICE" = "/dev/ttyS${1}" ]
	then
	    CONFIGURED="1"
	    echo "Device ${DEVICE} is configured as an IP network interface."
	    echo "Present settings:"
	    echo "Speed = ${LINESPEED}"
	    echo "Local IP address = ${LOCALIP}"
	    if [ "$PTP" != "" ]
	    then
		echo "Point-to-Point"
		echo "Remote IP = ${REMOTEIP}"
	    fi
	fi
    done

else
echo "No."
CONFIGURED="1"
fi

# out fall-through if configs exist, but not for this port
if [ "${CONFIGURED}" = "" ]
then
echo "No."
fi

echo
echo "Press enter to continue."
read TEMP

# Now check if device has a raw configuration file
DEVICE=${DEVBAK}
echo
echo "Checking if ${DEVICE} is configured as a raw port..."

if [  -f /etc/sysconfig/serial${1}raw  ]
then
    CONFIGURED=1
    echo "Present settings:"
    echo

    . /etc/sysconfig/serial${1}raw

    echo -n "Linespeed: "
    echo $LINESPEED
    echo -n "Data bits: "
    echo $DATABITS
    echo -n "Stop bits: "
    echo $STOPBITS
    echo -n "Parity: "
    echo $PARITY
    echo -n "Flow control: "
    echo $FLOWCONTROL
else
    echo "No."
fi

# Now check if it's had any entries anywhere.  if no,
# regular serial port.

if [ "$CONFIGURED" = "" ]
then
    echo
    echo "Device ${DEVICE} has no preset boot configuration."
    echo "The system default is being used."
fi

echo
echo "Press enter to continue."
read TEMP

}


#-----------------------------------------------------
# Function to configure terminals
#----------------------------------------------------

serial_term_config()
{

    typeset -i k=0
    LINESPEED="0"
    TERMEMUL=""
    clear
    echo
    echo "Configuring terminal device ttyS${1}"
    DEVICE="ttyS${1}"
    ITLINE="$(grep $DEVICE /etc/inittab | grep -v mgetty)"

    for i in $ITLINE
    do

	if [ "${k}" = "3" ]
	then
	    LINESPEED=$i
	fi

	if [ "${k}" = "4" ]
	then
	    TERMEMUL=$i
	fi

	k=${k}+1

    done
 
    echo 
    echo "Current setting for terminal ${DEVICE}:"
    echo "Line Speed = ${LINESPEED}"
    echo "Terminal emulation = ${TERMEMUL}"
    echo
    echo -n "New line speed? "
    read LINESPEED
    echo -n "New terminal emulation (default = vt100)? "
    read TERMEMUL

    if [ "$TERMEMUL" = "" ]
    then
	TERMEMUL="vt100"
    fi

    echo 
    echo "New setting for terminal ${DEVICE}:"
    echo "Line Speed = ${LINESPEED}"
    echo "Terminal emulation = ${TERMEMUL}"
    echo
    
    echo -n "Update terminal server configuration for ttyS${1} (y/N) ?"
    read TEMP

    if [ "$TEMP" = "y" -o "$TEMP" = "Y" ]
    then
	echo "Updating terminal server configuration..."
	# remove any old terminal entry from inittab
	grep -v ttyS${1} /etc/inittab > /tmp/inittemp
	cp /tmp/inittemp /etc/inittab
	chmod 644 /etc/inittab
	rm /tmp/inittemp
	# remove any slip configuration file
	rm -rf /etc/sysconfig/serial${1}ip
	# remove any raw configuration script
	rm -rf /etc/sysconfig/serial${1}raw
	# Add the new terminal line
	echo S${1}:23456:respawn:/sbin/getty -L $DEVICE $LINESPEED $TERMEMUL >> /etc/inittab
	echo "Terminal configuration updated."
    else
	echo
	echo "Terminal configuration NOT updated."
    fi

sleep 1
}


#------------------------------------------------------
# Function to configure ordinary serial ports
#-----------------------------------------------------

serial_raw_config()
{

clear
# echo "Serial ports are automatically configured at boot time."
# echo "Individual port speeds are set by the programs using them"
# echo "(the gateway-cgi program, for example.)"
    clear
    echo "Configure raw serial port ttyS${1}"
    echo
    echo "Present settings:"
    echo

    LINESPEED=""
    DATABITS=""
    STOPBITS=""
    PARITY=""
    FLOWCONTROL=""


    if [ -f /etc/sysconfig/serial${1}raw ]
    then
	. /etc/sysconfig/serial${1}raw
    fi

    echo -n "Linespeed: "
    echo $LINESPEED
    echo -n "Data bits: "
    echo $DATABITS
    echo -n "Stop bits: "
    echo $STOPBITS
    echo -n "Parity: "
    echo $PARITY
    echo -n "Flow control: "
    echo $FLOWCONTROL

    echo
    echo -n "New Line speed? "
    read LINESPEED
    echo -n "Data bits (5-8)? "
    read DATABITS
    echo -n "Stop bits (1, 2)? "
    read STOPBITS
    echo -n "Parity (E, O, M, S, N)? "
    read PARITY
    echo -n "Flow control (H, S, 4, N)? "
    read FLOWCONTROL

    echo
    echo "New settings:"
    echo -n "Linespeed: "
    echo $LINESPEED
    echo -n "Data bits: "
    echo $DATABITS
    echo -n "Stop bits: "
    echo $STOPBITS
    echo -n "Parity: "
    echo $PARITY
    echo -n "Flow control: "
    echo $FLOWCONTROL
    echo

    echo "Note that saving these changes will also remove all"
    echo "terminal configuration and IP port configuration for" 
    echo "device ttyS${1}.  Do you wish to make these modifications (y/N) ?"
    read TEMP

    if [ "$TEMP" = "y" -o "$TEMP" = "Y" ]
    then
	echo "Deleting terminal server configuration..."
	# remove any terminal entry from inittab
	grep -v ttyS${1} /etc/inittab > /tmp/inittemp
	cp /tmp/inittemp /etc/inittab
	rm /tmp/inittemp
	# remove any SLIP configuration file
	echo "Deleting serial IP configuration..."
	rm -rf /etc/sysconfig/serial${1}ip
	# remove any old configuration file
	rm -rf /etc/sysconfig/serial${1}raw
	echo "Setting new raw configuration..."
	# Now create the script...
	echo "#!/bin/bash" > /etc/sysconfig/serial${1}raw
	echo >> /etc/sysconfig/serial${1}raw
	echo "DEVICE=/dev/ttyS${1}" >> /etc/sysconfig/serial${1}raw
	echo -n "LINESPEED=" >> /etc/sysconfig/serial${1}raw
	echo $LINESPEED >> /etc/sysconfig/serial${1}raw
	echo -n "DATABITS=" >> /etc/sysconfig/serial${1}raw
	echo $DATABITS >> /etc/sysconfig/serial${1}raw
	echo -n "STOPBITS=" >> /etc/sysconfig/serial${1}raw
	echo $STOPBITS >> /etc/sysconfig/serial${1}raw
	echo -n "PARITY=" >> /etc/sysconfig/serial${1}raw
	echo $PARITY >> /etc/sysconfig/serial${1}raw
	echo -n "FLOWCONTROL=" >> /etc/sysconfig/serial${1}raw
	echo $FLOWCONTROL >> /etc/sysconfig/serial${1}raw
	echo >> /etc/sysconfig/serial${1}raw
	echo "Serial configuration updated."
    else
	echo
	echo "Serial configuration NOT updated."
    fi

sleep 1

}


#------------------------------------------------------
# Function to configure serial IP ports
#------------------------------------------------------

serial_ip_config()
{

    clear
    echo
    echo "Configuring serial device ttyS${1}:"
    echo

    LINESPEED=0
    LOCALIP=0.0.0.0
    PTP=""
    REMOTEIP=""
    DEVICE="/dev/ttyS${1}"

    if [ -f /etc/sysconfig/serial${1}ip ]
    then
	. /etc/sysconfig/serial${1}ip
    fi

    echo Present settings for device ${DEVICE}:
    echo Speed = $LINESPEED
    echo Local IP address = $LOCALIP
    if [ "$PTP" != "" ]
    then
	echo Point-to-Point
	echo Remote IP = $REMOTEIP
    fi

    echo
    echo

    echo -n "New line speed? "
    read LINESPEED
    echo -n "New Local IP address? "
    read LOCALIP
    echo -n "Point to Point (y/N)? "
    read PTP
    
    if [ "$PTP" = "y" -o "$PTP" = "Y" ]
    then
	PTP="pointopoint"
	echo -n "New remote IP address? "
	read REMOTEIP
    else
	PTP=""
    fi

    echo
    echo
    echo "New parameters for serial device ${DEVICE}:"
    echo Speed = $LINESPEED
    echo Local IP address = $LOCALIP
    if [ "$PTP" != "" ]
    then
	echo Point-to-Point
	echo Remote IP = $REMOTEIP
    fi

    echo
    echo
    echo -n "Accept these parameters (y/N)?"
    read ACCEPT

    if [ "$ACCEPT" = "y" -o "$ACCEPT" = "Y" ]
    then
	echo \#\!/bin/bash > /etc/sysconfig/serial${1}ip
	echo DEVICE=$DEVICE >> /etc/sysconfig/serial${1}ip
	echo LINESPEED=$LINESPEED >> /etc/sysconfig/serial${1}ip
	echo LOCALIP=$LOCALIP >> /etc/sysconfig/serial${1}ip
	echo PTP=$PTP >> /etc/sysconfig/serial${1}ip
	echo REMOTEIP=$REMOTEIP >> /etc/sysconfig/serial${1}ip
	chmod 754 /etc/sysconfig/serial${1}ip
	echo
	echo "Serial IP data updated."
	# remove any terminal entry from inittab
	grep -v ttyS${1} /etc/inittab > /tmp/inittemp
	cp /tmp/inittemp /etc/inittab
	rm /tmp/inittemp
	# remove any raw configuration file
	rm -rf /etc/sysconfig/serial${1}raw
    else
	echo
	echo "Serial IP data NOT updated."
    fi

sleep 1
}


#------------------------------------------------------------
# Functions to get and write parameters for ethernet devices
#------------------------------------------------------------
# Function to set ethernet parameters

eth_config()
{

    clear
    echo
    echo "Configuring network device eth${1}:"
    echo

    IF="eth${1}"
    IPADDR=
    BROADCAST=
    NETMASK=
    NETWORK=

    if [ -f /etc/sysconfig/eth${1}ip ]
    then
	. /etc/sysconfig/eth${1}ip
    fi

    echo Present settings for device ${IF}:
    echo Local IP address = $IPADDR
    echo Netmask = $NETMASK
    echo Broadcast address = $BROADCAST
    echo Network address = $NETWORK

    echo
    echo

    echo -n "New IP address (enter \"DHCP\" for DHCP dynamic IP addressing)? "
    read IPADDR
    echo -n "New Netmask? "
    read NETMASK
    echo -n "New broadcast address? "
    read BROADCAST
    echo -n "New network address? "
    read NETWORK
 

    echo
    echo
    echo "New parameters for network device ${IF}:"
    echo Local IP address = $IPADDR
    echo Netmask = $NETMASK
    echo Broadcast address = $BROADCAST
    echo Network address = $NETWORK

    echo
    echo
    echo -n "Accept these parameters (y/N)?"
    read ACCEPT

    if [ "$ACCEPT" = "y" -o "$ACCEPT" = "Y" ]
    then
	echo \#\!/bin/bash > /etc/sysconfig/eth${1}ip
	echo IF=$IF >> /etc/sysconfig/eth${1}ip
	echo IPADDR=$IPADDR >> /etc/sysconfig/eth${1}ip
	echo BROADCAST=$BROADCAST >> /etc/sysconfig/eth${1}ip
	echo NETWORK=$NETWORK >> /etc/sysconfig/eth${1}ip
	echo NETMASK=$NETMASK >> /etc/sysconfig/eth${1}ip
	chmod 754 /etc/sysconfig/eth${1}ip
	echo
	echo "Network device configured."
    else
	echo
	echo "Network device NOT configured."
    fi

sleep 1
}

#------------------------------------------------------------
# Function to read ethernet parameters

eth_view_info()
{
    clear
    echo
    echo "Configuration of network device eth${1}:"
    echo

    IF="eth${1}"
    IPADDR=""
    BROADCAST=""
    NETMASK=""
    NETWORK=""

    if [ -f /etc/sysconfig/eth${1}ip ]
    then
	. /etc/sysconfig/eth${1}ip
    fi

    echo Present settings for device ${IF}:
    echo Local IP address = $IPADDR
    echo Netmask = $NETMASK
    echo Broadcast address = $BROADCAST
    echo Network address = $NETWORK

    if [ "$IPADDR" = "" ]
    then
	echo
	echo "(${IF} is not a configured network device.)"
    fi

    echo
    echo
    echo "Press enter to continue."
    read TEMP
}

#-----------------------------------------------------------
# Menu function for ethernet configuration

enet_config()
{

while [ 1 ]; do

clear
echo
echo "                SIB network device configuration"
echo	
echo
echo "      1 ) Configure ethernet device"
echo "      2 ) View network device setting"
echo
echo "  b) Back to main menu"
echo "  ----------------------------------------------------------------------------"
echo -n "	Selection: "

read OPT


	case $OPT in
		b | B)	echo -e "\nReturning to main menu...\n"
			break ;;

		    1 )	echo -en "\n\n\tDevice (eth*) number to configure (starting from 0)? "
		        read PORTNUM
			eth_config  $PORTNUM
			;;

		    2 ) echo -en "\n\n\tDevice (eth*) number to view (starting from 0)? "
		        read PORTNUM
			eth_view_info $PORTNUM
			;;

	esac

done

}


#------------------------------------------------------------
# Functions to get and set routing data
#------------------------------------------------------------
# Function to add routes / nets

add_route()
{
# add a /sbin/route command to /etc/sysconfig/setroutes.sh via >> 
    clear
    echo "Add host / net to routing table"
    echo
    echo
    echo -n "Add host or net (H/n)? "
    read TYPE

    if [ "${TYPE}" = "n" -o "${TYPE}" = "N" ]
    then
	TYPE="net"
    else
	TYPE="host"
    fi

    echo -n "IP address of ${TYPE}? "
    read IPADDR
    echo -n "Netmask for ${TYPE} (optional)? "
    read NETMASK 
    echo -n "Network device used to access ${TYPE} (eg sl0 or eth0)? "
    read DEVICE

    TYPE="-${TYPE}"
    if [ "${NETMASK}" != "" ]
    then
	NETMASK="netmask ${NETMASK}"
    fi

    ROUTELINE="/sbin/route add ${TYPE} ${IPADDR} ${NETMASK} dev ${DEVICE}"

    echo 
    echo "Add the following entry into the routing table?"
    echo
    echo ${ROUTELINE}
    read CONFIRM

if [ "${CONFIRM}" = "y" -o "${CONFIRM}" = "Y" ]
then
	# add a routing entry

	if [ ! -f /etc/sysconfig/setroutes.sh ]
	then
	    echo \#\!/bin/bash > /etc/sysconfig/setroutes.sh
	    echo " " >> /etc/sysconfig/setroutes.sh
	    chmod 754 /etc/sysconfig/setroutes.sh
	fi

	echo ${ROUTELINE} >> /etc/sysconfig/setroutes.sh
	echo "Routing table updated."
else
    echo
    echo "Routing table NOT modified."
fi

sleep 1


}


#-----------------------------------------------------------
# Function to delete routes / nets

delete_route()
{

clear
echo "Delete hosts / nets"
echo
echo "$(grep -v /bin/bash /etc/sysconfig/setroutes.sh)"
echo
echo -n "IP address of host or route to delete? "
read IPADDRESS
echo
echo "Remove the following entry from the routing table (y/N)? "
echo
echo "$(grep ${IPADDRESS} /etc/sysconfig/setroutes.sh)"
read CONFIRM

if [ "${CONFIRM}" = "y" -o "${CONFIRM}" = "Y" ]
then
	# remove a routing entry
	grep -v ${IPADDRESS} /etc/sysconfig/setroutes.sh > /tmp/setroutes.tmp
	cp /tmp/setroutes.tmp /etc/sysconfig/setroutes.sh
	chmod 754 /etc/sysconfig/setroutes.sh
	rm /tmp/setroutes.tmp
	echo
	echo "Routing table updated."
else
    echo
    echo "Routing table NOT modified."
fi

sleep 1

}


#---------------------------------------------------------
# Function to add the default gateway

set_gateway()
{

    clear
    echo "Set default gateway."
    echo "Note that for your server to be able to access"
    echo "This gateway, you must have previously specified"
    echo "a route to the gateway."
    echo

    echo "Current routing information:"
    echo
    echo "$(grep default /etc/sysconfig/setroutes.sh)"
    echo
    
    echo
    echo -n "IP adress of default gateway? "
    read GATEWAY

    echo
    echo -n "Replace previous default gateway with new gateway ${GATEWAY} (y/N)? "
    read CONFIRM

    if [ "${CONFIRM}" = "y" -o "${CONFIRM}" = "Y" ]
    then
	# remove the old gateway entry
	grep -v default /etc/sysconfig/setroutes.sh > /tmp/setroutes.tmp
	cp /tmp/setroutes.tmp /etc/sysconfig/setroutes.sh
	chmod 754 /etc/sysconfig/setroutes.sh
	rm /tmp/setroutes.tmp
	# add the new gateway entry
	echo "/sbin/route add default gw ${GATEWAY} metric 1" >> /etc/sysconfig/setroutes.sh
	echo
	echo "Default gateway updated."
    else
	echo
	echo "Default gateway information NOT modified."
    fi

sleep 1

}


#----------------------------------------------------------
# Function to view routing data

view_routes()
{

    clear
    echo "Present routing table:"
    echo
    echo "$(grep -v /bin/bash /etc/sysconfig/setroutes.sh)"
    echo
    echo
    echo "Press enter to continue."
    read TEMP

}


#----------------------------------------------------------
# Routing configuration menu

route_config()
{

while [ 1 ]; do

clear
echo
echo "                SIB routing table configuration"
echo	
echo
echo "      1 ) Add routing entry"
echo "      2 ) Delete routing entry"
echo "      3 ) Set default gateway"
echo "      4 ) View routing data"
echo
echo "  b) Back to main menu"
echo "  ----------------------------------------------------------------------------"
echo -n "	Selection: "

read OPT


	case $OPT in
		b | B)	echo -e "\nReturning to main menu...\n"
			break ;;

		    1 ) add_route
			;;

		    2 ) delete_route
			;;

		    3 ) set_gateway
		        ;;

		    4 ) view_routes
		        ;;

	esac
			

done

}


#-----------------------------------------------------------
# Hostname configuration function
#-----------------------------------------------------------

hostname_config()
{
    clear
    echo "Host Name Configuration"
    echo
    echo -n "Current host name:  "
    echo "$(cat /etc/hostname)"
    echo
    echo
    echo -n "Change host name (y/N)? "
    read CONFIRM

    if [ "${CONFIRM}" = "y" -o "${CONFIRM}" = "Y" ]
    then
	echo
	echo -n "Enter new host name:  "
	read HOSTNAME
	echo
	echo -n "Change hostname to ${HOSTNAME} ? "
	read TEMP

	    if [ "${TEMP}" = "y" -o "${TEMP}" = "Y" ]
		then
		    echo "${HOSTNAME}" > /etc/hostname
		    chmod 644 /etc/hostname
		    echo
		    echo "Hostname changed."
		else
		    echo
		    echo "Hostname NOT changed."
		fi
    else
	echo
	echo "Returning to main menu..."
    fi

sleep 1

}


#-----------------------------------------------------------
# Name resolution (hosts and resolv.conf) config functions
#-----------------------------------------------------------
# Function to add a nameserver

add_nameserver()
{
    clear
    echo "Add a name server"
    echo
    echo "Current name server information: "
    echo
    echo "$(grep -v \# /root/resolv.conf)"
    echo
    echo "(This should contain a single line containing the \"search\""
    echo " identifier and your domain name, plus as many \"nameserver\""
    echo " line entries as you have nameservers.)"
    echo
    echo -n "Add search entry or nameserver entry (s/N)? "
    read NORS

    if [ "${NORS}" = "s" -o "${NORS}" = "S" ]
    then
	NORS="search"
	echo -n "Domain to search (your domain)?  "
    else
	NORS="nameserver"
	echo -n "Numeric IP address of nameserver?  "
    fi

    read IPADDR
    echo
    echo "Add the following entry to your nameserver configuration?"
    echo
    echo "${NORS}     ${IPADDR}"

    read CONFIRM

    if [ "${CONFIRM}" = "y" -o "${CONFIRM}" = "Y" ]
    then
	echo "${NORS}     ${IPADDR}" >> /root/resolv.conf
	echo
	echo "${NORS} entry added to nameserver configuration."
    else
	echo
	echo "${NORS} entry NOT added."
    fi

sleep 1

}

#-------------------------------------------------------------
# Function to delete a nameserver

delete_nameserver()
{
    clear
    echo "Delete a name server"
    echo
    echo "Current name server information: "
    echo
    echo "$(grep -v \# /root/resolv.conf)"
    echo
    echo "(This should contain a single line containing the \"search\""
    echo " identifier and your domain name, plus as many \"nameserver\""
    echo " line entries as you have nameservers.)"
    echo
    echo -n "IP address of entry to delete?  "
    read IPADDR

    echo
    echo
    echo "Delete the following line from your nameserver configuration?"
    echo "$(grep ${IPADDR} /root/resolv.conf)"

    read CONFIRM

    if [ "${CONFIRM}" = "y" -o "${CONFIRM}" = "Y" ]
    then
	grep -v ${IPADDR} /root/resolv.conf > /tmp/resolv.tmp
	cp /tmp/resolv.tmp /root/resolv.conf
	chmod 644 /root/resolv.conf
	rm /tmp/resolv.tmp
	echo
	echo "Name server entry deleted."
    else
	echo
	echo "Name server configuration NOT modified."
    fi

sleep 1

}


#------------------------------------------------------------
# Function to add an entry to /etc/hosts

add_host()
{

    clear
    echo "Add static host entry"
    echo
    echo "Current static host information: "
    echo
    echo "IP address          Host name            Alias"
    echo
    echo "$(grep -v localhost /etc/hosts)"
    echo
    echo
    echo -n "Host name to add?  "
    read HOSTNAME
    echo -n "IP address of ${HOSTNAME}?  "
    read IPADDR
    echo -n "Alias for ${HOSTNAME} (optional)?  "
    read ALIAS
    echo
    echo "Add the following entry to your nameserver configuration?"
    echo
    echo "${IPADDR}     ${HOSTNAME}     ${ALIAS}"
    read CONFIRM

    if [ "${CONFIRM}" = "y" -o "${CONFIRM}" = "Y" ]
    then
	echo "${IPADDR}     ${HOSTNAME}     ${ALIAS}" >> /etc/hosts
	echo "Static host configuration updated."
    else
	echo
	echo "Static host entry NOT added."
    fi

sleep 1

}


#-------------------------------------------------------------
# Function to delete an entry from /etc/hosts

delete_host()
{
    clear
    echo "Delete a static host entry"
    echo
    echo "Current static host information: "
    echo
    echo "IP address          Host name            Alias"
    echo
    echo "$(grep -v localhost /etc/hosts)"
    echo
    echo -n "Name, IP address, or alias of entry to delete?  "
    read IPADDR

    echo
    echo
    echo "Delete the following line from your static host configuration?"
    echo "$(grep ${IPADDR} /etc/hosts)"

    read CONFIRM

    if [ "${CONFIRM}" = "y" -o "${CONFIRM}" = "Y" ]
    then
	grep -v ${IPADDR} /etc/hosts > /tmp/hosts.tmp
	cp /tmp/hosts.tmp /etc/hosts
	chmod 644 /etc/hosts
	rm /tmp/hosts.tmp
	echo
	echo "Host entry deleted."
    else
	echo
	echo "Host configuration NOT modified."
    fi

sleep 1

}


#------------------------------------------------------------
# Function to view /etc/hosts and /root/resolv.conf

view_dns()
{
    clear
    echo "View current static host and name server configuration"
    echo
    echo
    echo "Current static host information: "
    echo
    echo "IP address          Host name            Alias"
    echo
    echo "$(grep -v localhost /etc/hosts)"
    echo
    echo
    echo "Current name server information: "
    echo
    echo "$(grep -v \# /root/resolv.conf)"
    echo
    echo "(This should contain a single line containing the \"search\""
    echo " identifier and your domain name, plus as many \"nameserver\""
    echo " line entries as you have nameservers.)"
    echo
    echo
    echo "Press enter to return..."
    read TEMP

}


#-----------------------------------------------------------
# Name resolution main menu

dns_config()
{

while [ 1 ]; do

clear
echo
echo "                SIB name resolution configuration menu"
echo	
echo
echo "      1 ) Add nameserver"
echo "      2 ) Delete nameserver"
echo "      3 ) Add static host entry"
echo "      4 ) Delete static host entry"
echo "      5 ) View name resolution configuration"
echo
echo "  b) Back to main menu"
echo "  ----------------------------------------------------------------------------"
echo -n "	Selection: "

read OPT


	case $OPT in
		b | B)	echo -e "\nReturning to main menu\n"
			break ;;
		    1 )	add_nameserver
			;;
		    2 )	delete_nameserver  
			;;
		    3 )	add_host
			;;
		    4 )	delete_host  
			;;
		    5 )	view_dns
			;;

	esac
			

done


}


#----------------------------------------------------------
# Function to activate changes
#----------------------------------------------------------

activate_changes()
{
   /etc/sysconfig/activate_config.sh
}


#----------------------------------------------------------
# Function to change root's password
#----------------------------------------------------------

change_pword()
{
    clear
    echo "Change root password."
    echo
    echo
    echo -n "Do you really want to do this (y/N)?  "
    read CONFIRM

    if [ "${CONFIRM}" = "y" -o "${CONFIRM}" = "Y" ]
    then
	/usr/bin/passwd root  
	# add in anything here for web configuration...
    else
	echo 
	echo "Password will not be changed."
    fi

sleep 1
}

#------------------------------------------------------------
# Function to confirm shutdown and type of shutdown
#------------------------------------------------------------

confirm_shutdown()
{
    clear
    echo "Shutdown SIB"
    echo 
    echo -n "Halt or Reboot (h/R)? "
    read SHUTDOWNTYPE
    echo

    if [ "${SHUTDOWNTYPE}" = "h" -o "${SHUTDOWNTYPE}" = "H" ]
    then
	echo -n "Halt"
	SHPARAM="-h"
    else
	echo -n "Reboot"
	SHPARAM="-r"
    fi

    echo " system.  Do you wish to proceed (y/N)? "
    read CONFIRM

    if [ "${CONFIRM}" = "y" -o "${CONFIRM}" = "Y" ]
    then
	echo "Proceeding with shutdown process."
	/sbin/shutdown ${SHPARAM} now
	# The above should work, since this script should be SUID root...
    else
	echo "Shutdown aborted."
    fi

sleep 1
}


#-----------------------------------------------------------
# Boot-time configuration (modules, read-only, etc.)

boot_config()
{

while [ 1 ]; do

clear
echo
echo "                SIB boot parameter configuration menu"
echo	
echo
echo "      1 ) Set root filesystem READ-ONLY"
echo "      2 ) Set root filesystem READ-WRITE"
echo "      3 ) Configure loadable modules / drivers"
echo
echo "  b) Back to main menu"
echo "  ----------------------------------------------------------------------------"
echo -n "	Selection: "

read OPT


	case $OPT in
		b | B)	echo -e "\nReturning to main menu\n"
			break ;;
		    1 )
			if [ -f /etc/READONLY ]
			then
			    :
			else
			    echo "readonly" > /etc/READONLY
			    chmod 664 /etc/READONLY
			fi
			echo -e "\n\n\tFilesystem will be mounted READ-ONLY on next reboot."
			sleep 1
			;;
		    2 )	rm -rf /etc/READONLY
			echo -e "\n\n\tFilesystem will be mounted READ-WRITE on next reboot."
			sleep 1
			;;
		    3 )	pushd ./
			cd /etc
			nano modules
			popd
			;;


	esac
			

done


}


#-----------------------------------------------------------
# Main configuration menu function
#----------------------------------------------------------

# This is necessary in case they use the READ-ONLY option
(cat /proc/mounts|grep root;echo) | while read dev mp fs RDO parm
do

if [ "${dev}" = "/dev/root" -a "${RDO}" = "ro" ]
    then
	echo true > /tmp/READONLY
	echo "Filesystem mounted read-only, remounting read-write..."
	mount -o remount,rw /dev/diskroot /
	sleep 2
    fi

done


while [ 1 ]; do

clear
echo
echo "                Server in a Box -- main configuration menu"
echo	
echo
echo "      1 ) Serial settings"
echo "      2 ) Ethernet settings"
echo "      3 ) Routing tables"
echo "      4 ) Host name"
echo "      5 ) Internet name resolution"
echo "      6 ) Configure periodic command scheduler"
echo "      7 ) Configure boot-time parameters"
if [ -f /usr/lib/cgi-bin/gateway-cgi ]
then
echo "      8 ) Edit gateway-cgi configuration file"
fi
echo
echo "  B ) Bash prompt"
echo "  T ) Set time and date"
echo "  C ) Change root password" 
echo "  L ) CAN loopback Test"

#if [ "${USER}" = "root" ] 
#then
#    echo "  A ) Activate changes"
#fi

echo "  S ) Shutdown SIB"
echo "  q ) quit"
echo "  ----------------------------------------------------------------------------"
echo -n "	Selection: "

read OPT


	case $OPT in
		    q | Q )	echo
			echo "Exiting configuration menu."
			if [ "${USER}" = "root" ] 
			then
			    echo "If you did not activate changes,"
			fi
			echo "Your changes will be activated the next time the system restarts."
			echo
			break ;;
		    1 )	serial_config
			;;
		    2 )	enet_config  
			;;
		    3 ) route_config  
			;;
		    4 ) hostname_config
			;;
		    5 ) dns_config
			;;
		    6 ) pushd ./ 
			cd /etc 
			nano crontab
			popd
			;;
		    7 ) boot_config
		        ;;
		    8 ) if [ -f /usr/lib/cgi-bin/gateway-cgi ]
			then
			    pushd ./ 
			    cd /usr/lib/cgi-bin 
			    nano cgi.config 
			    popd
			fi
			;;

		    t | T ) time_and_date
			;;
		    c | C ) change_pword
			;;

		    b | B ) echo; echo "Type \"exit\" to return to the configuration utility."
			    /bin/bash
			    ;;

#		    a | A )  if [ "${USER}" = "root" ] 
#			     then
#				activate_changes
#			     fi
#			;;
		    s | S )  confirm_shutdown
			;;
			
		    l | L ) /can/canstart.sh
			    /can/CanOpener/loopback
		            sleep 7
			;;

	esac
			

done

if [ -f /tmp/READONLY ]
then
    echo "Remounting filesystem read-only..."
    rm -rf /tmp/READONLY
    mount -o remount,ro /dev/diskroot /
fi

sleep 2

exit 0








